#	include	"Simulator.h"

// "Simple" is a simple version of the sim3 simulator which is used
// mainly for testing -- see sim3mp for the full thing.


Simulator	*theSimulator;
int
main (
	int		argc,
	char	*argv[],
	char	*argp[])
	{
	// usage: sim3 program [args..]

	if(argc<2)
		{
		printf("usage: %s program [args..]\n", argv[0]);
		return(1);
		}
	// Global Intialization
	InitializeSimMaskToOpCodeTable();


	// Initialize system-board devices
	theSimulator = new Simulator;
	StandardDRAMDevice			*DRAM = new StandardDRAMDevice;
	SingleIssueProcessor		*Proc = new SingleIssueProcessor;
	SystemTrapHandler			*STH = new SystemTrapHandler;
	DisembodiedCache			*L1I = new DisembodiedCache;
	DisembodiedCache			*L1D = new DisembodiedCache;
	DisembodiedCache			*L2 = new DisembodiedCache;

	theSimulator->SetName("Machine");
	DRAM->SetName("DRAM");
	Proc->SetName("Processor");
	STH->SetName("OS");
	L1I->SetName("L1-I");
	L1D->SetName("L1-D");
	L2->SetName("L2");

	DRAM->Initialize(0x80000, 0x1000, 60, 10);	// init 2 gig space
	L1I->Initialize(128, 32, 4, false, 0, 1, 0);	// 16k
	L1D->Initialize(128, 64, 2, false, 0, 1, 0);	// 16k
	L2->Initialize(128, 2048, 4, true, 4, 2, 10);	// 1meg

	// Connect system board devices together
	L1I->ConnectToUpStreamComponent(Proc);
	Proc->ConnectDataMemory(L1D);
	L1D->ConnectToUpStreamComponent(Proc);
	Proc->ConnectInstructionMemory(L1I);
	Proc->ConnectSystemTrapHandler(STH);
	L1I->ConnectToDownStreamComponent(L2);
	L1D->ConnectToDownStreamComponent(L2);
	L2->ConnectToUpStreamComponent(Proc);
	L2->ConnectToDownStreamComponent(DRAM);
	DRAM->Connect(L2);

	// Print the system board devices configuration	
	theSimulator->PrintConfiguration();
	Proc->PrintConfiguration();
	STH->PrintConfiguration();
	L1I->PrintConfiguration();
	L1D->PrintConfiguration();
	L2->PrintConfiguration();
	DRAM->PrintConfiguration();

	if(!Loader::LoadFromDevice(
		argv[1], DRAM, Proc, &argv[1], argp))
		{
		printf("failed to load %s into memory\n", argv[1]);
		return(2);
		}

	theSimulator->RegisterComponent(theSimulator);
	theSimulator->RegisterComponent(DRAM);
	theSimulator->RegisterComponent(Proc);
	theSimulator->RegisterComponent(STH);
	theSimulator->RegisterComponent(L1I);
	theSimulator->RegisterComponent(L1D);
	theSimulator->RegisterComponent(L2);

	theSimulator->Simulate();

	printf("\n");
	theSimulator->PrintStatistics();
	Proc->PrintStatistics();
	STH->PrintStatistics();
	L1I->PrintStatistics();
	L1D->PrintStatistics();
	L2->PrintStatistics();
	DRAM->PrintStatistics();
	
	delete Proc;
	delete DRAM;
	delete STH;
	delete L1I;
	delete L1D;
	delete L2;
	return(0);	
	}


